/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Foundation/Foundation.h>
#import "MontageInterface.h"
#ifndef WorkingCodec_h
#include "WorkingCodec.h"
#endif //WorkingCodec_h

extern NSString *MIHighQualityHasChangedNotification;
extern NSString *MISingleFieldHasChangedNotification;


enum
{
	kPrefsSnapHGrid = 1,
	kPrefsSnapHObjects = 2,
	kPrefsSnapHLocators = 4,
	kPrefsSnapVGrid = 8,
	kPrefsSnapVObjects = 16
};

enum
{
	kPrefsGridShowVLines = 1,
	kPrefsGridShowHLines = 2,
	kPrefsGridShowVObjLines = 4,
	kPrefsGridShowHObjLines = 8
};

enum
{
	kPrefsGridsVSpacingVerySmall = 0,
	kPrefsGridsVSpacingSmall = 1,
	kPrefsGridsVSpacingMedium = 2,
	kPrefsGridsVSpacingBig = 3
};

enum
{
	kPrefsPerfThumbsWhenPlaying = 1 << 0,
	kPrefsPerfObjectShadows = 1 << 1,
	kPrefsPerfUniformBackground = 1 << 2,
	kPrefsPerfMovingTitles = 1 << 3,
};

enum
{
	kPrefsStackingMethodVertically = 0,
	kPrefsStackingMethodHorizontally,
	kPrefsStackingMethodAlwaysAsk,
	kPrefsStackingMethodCancelled
};

enum
{
	kTimeDisplaySMPTE = 1,
	kTimeDisplayBPM = 2
};

@class MITime;

@interface MIPreferences : NSObject 
{
	NSString*	fHyperEngineAVSerial;
    NSString*	fRayGunProSerial;
    NSString*	fRayGunSerial;
    NSString*	fHyperprismSilverSerial;
    NSString*	fHyperprismGoldSerial;

	// Snapping pane
	UInt32	fSnapOptionsMask;
//	UInt32	fShiftSnapOptionsMask;
	
	// Grid pane
	UInt32	fGridShowMask;
	UInt32	fGridVSpacing;

	// performance pane
	UInt32	fPerfOptionsMask;

	// Undo Stack Depth
	SInt32	fUndoStackDepth;
	
	// stacking method
	SInt32	fStackingMethod;
	
	// SMPTE or BPM display
	SInt32	fTimeDisplay;
	
	// workspace bkgnd color
	NSColor	*fWorkspaceBGColor;
	
	// audio devices	
	NSString*	fAudioOutputDeviceStringCode;
	NSString*	fAudioInputDeviceStringCode;
	UInt32		fAudioOutputDeviceBufferNbFrames;
	
	BOOL		fAskNewDocProperties;
	UInt32		fHorizontalGridSpacing;
	//UInt32		fFPS;
	UInt32		fTempo;
	UInt32		fSignatureNum;
	UInt32		fSignatureDen;
	WorkingCodecType		fPreviewCodec;
	WorkingCodecType		fRenderCodec;

	// Text Clips
	BOOL		fFixedDuration;
	float		fDefaultDuration;
	float		fScrollSpeed;

	// Still Clips
	float		fStillDefaultDuration;

	// Playback Quality
	BOOL		fHighQuality;
	BOOL		fSingleField;
	
	// reference external files
	BOOL		fReferenceExternalFiles;
  
	NSCalendarDate*		fFirstRun;
	
  // transitions
  id				fDefaultTransition;
  BOOL				fAutomaticTransitionParameters;
  QTAtomContainer	fDefaultTransitionEffectDesc;
}

+(MIPreferences*)sharedPrefs;

-(bool)isSerialized;
-(void)setSerialized;

-(void)setSerial:(NSString*)serial forProduct:(int)product;
-(const char*)getSerialFor:(int)product;

-(UInt32)snapOptionsMask;
-(void)setSnapOptionsMask:(UInt32)mask;

// -(UInt32)shiftSnapOptionsMask;
// -(void)setShiftSnapOptionsMask:(UInt32)mask;

-(UInt32)gridShowMask;
-(void)setGridShowMask:(UInt32)mask;

-(UInt32)gridVSpacingIndex;
-(void)setGridVSpacing:(UInt32)spacing;

- (void)setUndoStackDepth:(SInt32)depth;
- (SInt32)undoStackDepth;

- (void)setStackingMethod:(SInt32)stackingMethod;
- (SInt32)stackingMethod;

- (void)setTimeDisplay:(SInt32)td;
- (SInt32)timeDisplay;

- (void)setWorkspaceBGColor:(NSColor*)color;
- (NSColor*)workspaceBGColor;
	
- (void)setAskNewDocProperties:(BOOL)val;
- (BOOL)askNewDocProperties;

- (void)setHorizontalGridSpacing:(UInt32)val;
- (UInt32)getHorizontalGridSpacing;

//- (void)setDefaultFPS:(UInt32)val;
//- (UInt32)getDefaultFPS;

- (void)setDefaultTempo:(UInt32)val;
- (UInt32)getDefaultTempo;

- (void)setDefaultSignatureNum:(UInt32)val;
- (UInt32)getDefaultSignatureNum;

- (void)setDefaultSignatureDen:(UInt32)val;
- (UInt32)getDefaultSignatureDen;


- (void)setDefaultRenderFormat:(WorkingCodecType)codec;
- (void)setDefaultPreviewFormat:(WorkingCodecType)codec;
- (WorkingCodecType)defaultRenderFormat;
- (WorkingCodecType)defaultPreviewFormat;





// ASCII string code that describe the audio deviuce
- (NSString*)audioOutputDeviceStringCode;
- (NSString*)audioInputDeviceStringCode;
- (UInt32)audioOutputDeviceBufferNbFrames;
// ASCII string code that describe the audio deviuce
- (void)setAudioOutputDeviceStringCode:(NSString*)code;
- (void)setAudioInputDeviceStringCode:(NSString*)code;
- (void)setAudioOutputDeviceBufferNbFrames:(UInt32)nbFrames;

// in pixels
-(MIVPos)gridVSpacing;

-(UInt32)perfOptionsMask;
-(void)setPerfOptionsMask:(UInt32)mask;
+(BOOL)isGenerateThumbnailsWhenPlayingDoc;
+(BOOL)shadowsAroundObject;
+(BOOL)uniformBackground;
+(BOOL)movingTitles;

+(void)addDefaultValuesToDictionary:(NSMutableDictionary*)appDefaults;
-(void)saveToUserSettings;
-(void)restoreFromUserSettings;

// Text Clip

- (BOOL)isFixedDuration;
- (void)setIsFixedDuration:(BOOL)flag;	

- (void)setTextClipDefaultDuration:(float)duration;
- (float)textClipDefaultDuration;

- (void)setTextClipScrollSpeed:(float)speed;
- (float)textClipScrollSpeed;

- (void)setStillClipDefaultDuration:(float)duration;
- (float)stillClipDefaultDuration;

- (void)setIsHighQuality:(BOOL)flag;
- (BOOL)isHighQuality;

- (void)setIsSingleField:(BOOL)flag;
- (BOOL)isSingleField;

- (void)setReferenceExternalFiles:(BOOL)flag;
- (BOOL)referenceExternalFiles;

- (void)setAutomaticTransitionParameters:(BOOL)flag;
- (BOOL)automaticTransitionParameters;

- (NSCalendarDate*)getFirstRunDate;

- (void)setDefaultTransition:(id)value;
- (id)getDefaultTransition;
- (QTAtomContainer)getDefaultTransitionEffectDesc;
+ (id)createDefaultTransition;
@end
